--[[---------------------------------------
	VARIABLE CACHING
-----------------------------------------]]

-- Lua Functions
local floor = math.floor
local pairs, select, tonumber, type, unpack = pairs, select, tonumber, type, unpack
local find, gsub, lower, sub = string.find, gsub, string.lower, string.sub
-- WoW API Functions / Variables
local CreateFont, CreateFrame = CreateFont, CreateFrame
local GetAddOnMetadata = GetAddOnMetadata
local GetSpellInfo = GetSpellInfo
local GetSpecialization, GetSpecializationInfo = GetSpecialization, GetSpecializationInfo
local IsShiftKeyDown, IsControlKeyDown = IsShiftKeyDown, IsControlKeyDown
local UnitClass = UnitClass

local SCA, Ace3, LSM = unpack(select(2,...))
Ace3.version = GetAddOnMetadata(..., 'Version')

--[[---------------------------------------
	CONSTANTS
-----------------------------------------]]
local SOUND_SOURCE_OPTIONS = {
	["packs"] = "SCA Audio Packs",
	["shared"] = "SharedMedia",
	["custom"] = "Custom Sounds",
}

local SOUND_CHANNEL_OPTIONS = {
	["Master"] = MASTER,
	["SFX"] = SOUND,
	["Music"] = MUSIC_VOLUME,
	["Ambience"] = AMBIENCE_VOLUME,
	["Dialog"] = DIALOG_VOLUME,
}

local SOUND_CUSTOM_TYPES = {
	["file"] = "Single Audio File",
	["pack"] = "Custom Audio Pack",
}

local SOUND_CRIT_TRIGGERS = {
	['crit'] = "Crits Only",
	['nocrit'] = "Non-crits Only",
	['always'] = "Always On",
}

local SOUND_EVENTS = {
	['onCast'] = "On Cast Start",
	['onSuccess'] = "On Cast Success",
	['onFail'] = "On Cast Fail",
	['onDamage'] = "On Damage Dealt",
	['onHeal'] = "On Heal Dealt",
}

local DEFENSE_EVENTS = {
	['absorb'] = COMBAT_TEXT_ABSORB,
	['block'] = COMBAT_TEXT_BLOCK,
	['dodge'] = COMBAT_TEXT_DODGE,
	['parry'] = COMBAT_TEXT_PARRY,
}

local MOVEMENT_ACTIONS = {
	['onStart'] = "On Action Start",
	['onStop'] = "On Action Stop",
}

local toggle_options

local function GetToggleOptions()
	SCA.temp.selectedMenuItem = 1
	if (not toggle_options) then
		local db = Ace3.db.char
		
		local damage = db.audio.spells.damage
		local heals = db.audio.spells.heals
		local specific = db.audio.spells.specific
		
		local defense = db.audio.defense
		local combat = db.audio.combat
		local looting = db.audio.looting
		
		local chat = db.audio.chat
		local move = db.audio.move
		
		toggle_options = {
			type = "group",
			order = 1,
			name = "",
			desc = '',
			guiInline = true,
			args = {
				DamageSpells = {
					name = "Damage Spells",
					order = 1,
					type = "group",
					args = Ace3:GroupBuilder(damage,"normal"),
				},
				HealingSpells = {
					name = "Healing Spells",
					order = 2,
					type = "group",
					args = Ace3:GroupBuilder(heals,"normal"),
				},
				SpecificSpells = {
					name = "Specific Spells",
					order = 3,
					type = "group",
					args = Ace3:GroupBuilder(specific,nil,true),
				},
				Defense = {
					name = "Defense Sounds",
					order = 4,
					type = "group",
					args = Ace3:GroupBuilder(defense,"half"),
				},
				Combat = {
					name = "Combat Sounds",
					order = 5,
					type = "group",
					args = Ace3:SubGroupBuilder(combat),
				},
				Looting = {
					name = "Looting Sounds",
					order = 5,
					type = "group",
					args = Ace3:SubGroupBuilder(looting),
				},
				Chat = {
					name = "Chat Sounds",
					order = 6,
					type = "group",
					args = Ace3:SubGroupBuilder(chat),
				},
				Movement = {
					name = "Movement Sounds",
					order = 6,
					type = "group",
					args = Ace3:SubGroupBuilder(move),
				},
			},
		}
	end
	
	return toggle_options
end

local spell_options

local function GetSpellOptions()
	SCA.temp.selectedMenuItem = 2
	if (not spell_options) then
		local db = Ace3.db.char
		
		local damage = db.audio.spells.damage
		local heals = db.audio.spells.heals
		local specific = db.audio.spells.specific
		local ignored = db.audio.spells.ignored
		
		local newSound = SCA.temp.newSound
		local newIgnore = SCA.temp.newIgnore
		
		spell_options = {
			type = "group",
			childGroups = "tab",
			order = 1,
			name = "",
			desc = '',
			args = {
				DamageTab = Ace:SpellTabBuilder(damage,"Damage","onDamage"),
				HealingTab = Ace3:SpellTabBuilder(heals,"Healing","onHeal"),
				SpecificTab = {
					name = "Specific Spells",
					order = 3,
					type = "group",
					args = {
						disabledError = Ace3:DisabledError(specific,0),
						AddCancelSound = {
							order = function()
								if (SCA.temp.addSound) then
									return 9
								else
									return 1
								end
							end,
							type = "execute",
							name = function(self)
								if (SCA.temp.addSound) then
									self.options.args.SpecificTab.args.AddCancelSound.width = "half"
								else
									self.options.args.SpecificTab.args.AddCancelSound.width = "normal"
								end
								
								if (SCA.temp.addSound) then
									return "Cancel"
								else
									return "Add Sound"
								end
							end,
							func = function()
								if (SCA.temp.addSound) then
									if (SCA.SpellFrame:IsShown()) then
										SCA.SpellFrame:Hide()
									end
									Ace3:ResetDB(newSound,true,true)
								else
									SCA.temp.addSound = true
								end
								GameTooltip:Hide()
							end,
							width = "normal",
						},
						SpellConfig = {
							name = "Spell Config",
							hidden = function()
								return not SCA.temp.addSound
							end,
							order = 2,
							type = "group",
							guiInline = true,
							args = {
								spellInfo = {
									type = "input",
									order = 1,				
									name = "Spell ID / Name",
									desc = "Input spell id, or name, to be assigned a specific sound.\n\nSpell names and IDs that you input are checked against a pre-built table of spell ID and names. If there is a spell that is missing that you feel should be added, please message me via the addon page.\n\nClicking the \"Spells\" button will display a list of spells that currently exist for your current class/spec.",
									disabled = function()
										if (SCA.temp.editSound) then
											return true
										else
											return false
										end
									end,
									get = function(self)
										return newSound.validSpell
									end,
									set = function(self,value)
										local errors = self.options.args.SpecificTab.args.errors.args.messages
										
										Ace3:CheckSpecificSpell(newSound,errors,Ace3:CheckSpell('onSuccess',value))
										
										Ace3:UpdateTutorial(newSound)
										
										-- If no event is selected ("empty"), we'll need to refresh the config's contents.
										if (newSound.selected == "empty") then
											Ace3:NavigateInterfaceOptions(SCA.temp.selectedMenuItem,true)
										end
									end,
								},
								spells = {
									order = 2,
									type = "execute",
									name = "Spells",
									desc = "List of spells, and spell IDs, that pertain to your class and spec.\n\n|cFFFF9B00Orange Spells|r are spells that are currently on the ignore list.\n\nA |cFF00FF00Green Spell|r is the spell that is currently selected.\n\n|cFF999999Grey Spells|r are spells that already have sounds assigned to them.",
									disabled = function()
										if (SCA.temp.editSound) then
											return true
										else
											return false
										end
									end,
									func = function()
										local _,class = UnitClass("player")
										local spec = GetSpecialization()
										if (SCA.SpellFrame:IsShown()) then
											SCA.SpellFrame:Hide()
										else
											for k,_ in pairs(SCA.spellNames[class][spec]) do
												if (SCA.SpellFrame[k]) then
													if (specific[SCA.spellNames[class][spec][k]]) then
														SCA.SpellFrame[k]:Disable()
														SCA.SpellFrame[k]:SetDisabledFontObject(SCA.SpellFrame.dFont)
													elseif (ignored[SCA.spellNames[class][spec][k]]) then
														if (not SCA.SpellFrame[k]:IsEnabled()) then
															SCA.SpellFrame[k]:Enable()
														end
														SCA.SpellFrame[k]:SetNormalFontObject(SCA.SpellFrame.cFont)
														SCA.SpellFrame[k]:SetHighlightFontObject(SCA.SpellFrame.CFont)
													elseif (k == newSound.validSpell) then
														if (not SCA.SpellFrame[newSound.validSpell]:IsEnabled()) then
															SCA.SpellFrame[newSound.validSpell]:Enable()
														end
														SCA.SpellFrame[newSound.validSpell]:SetNormalFontObject(SCA.SpellFrame.sFont)
														SCA.SpellFrame[newSound.validSpell]:SetHighlightFontObject(SCA.SpellFrame.sFont)
													else
														if (not SCA.SpellFrame[k]:IsEnabled()) then
															SCA.SpellFrame[k]:Enable()
														end
														SCA.SpellFrame[k]:SetNormalFontObject(SCA.SpellFrame.nFont)
														SCA.SpellFrame[k]:SetHighlightFontObject(SCA.SpellFrame.hFont)
													end
												end
											end
											SCA.SpellFrame:Show()
										end
									end,
									width = "half",
								},
								spacing1 = Ace3:Spacer(3,"half"),
								tutorial = Ace3:TutorialToggle(newSound,4,"normal"),
								onCast = Ace3:EventToggle(newSound,'onCast',5,newSound.errorName,"half","Start","Toggles whether a sound will occur when starting a spell cast.\n\nCombat Log Event: |cFF00FF00SPELL_CAST_START|r","SpecificTab",true),
								onSuccess = Ace3:EventToggle(newSound,'onSuccess',6,newSound.errorName,"half","Success","Toggles whether a sound will occur when a spell cast completes.\n\nCombat Log Event: |cFF00FF00SPELL_CAST_SUCCESS|r","SpecificTab",true),
								onFail = Ace3:EventToggle(newSound,'onFail',7,newSound.errorName,"half","Failed","Toggles whether a sound will occur when a spell cast fails.\n\nCombat Log Event: |cFF00FF00SPELL_CAST_FAILED|r","SpecificTab",true),
								onDamage = Ace3:EventToggle(newSound,'onDamage',9,newSound.errorName,"half","Damage","Toggles whether a sound will occur when starting a spell causes damage.\n\nCombat Log Events: |cFF00FF00SPELL_DAMAGE|r, |cFF00FF00SWING_DAMAGE|r, |cFF00FF00SPELL_PERIODIC_DAMAGE|r","SpecificTab",true),
								onHeal = Ace3:EventToggle(newSound,'onHeal',10,newSound.errorName,"half","Heal","Toggles whether a sound will occur when starting a spell causes healing.\n\nCombat Log Events: |cFF00FF00SPELL_HEAL|r, |cFF00FF00SPELL_PERIODIC_HEAL|r","SpecificTab",true),
							},
						},
						soundList = Ace3:SoundList(newSound,3,true),
						errors = Ace3:Errors(newSound.errorName,4,true),
						SoundConfig = {
							name = "Audio Config",
							disabled = function()
								if (newSound.validSpell and newSound.numEvents > 0) then
									return false
								else
									return true
								end
							end,
							hidden = function()
								return not SCA.temp.addSound
							end,
							order = 5,
							type = "group",
							guiInline = true,
							args = {
								source = Ace3:Source(newSound,1,newSound.errorName,SOUND_SOURCE_OPTIONS,true),
								channel = Ace3:Channel(newSound,2,SOUND_CHANNEL_OPTIONS,true),
								volume = Ace3:Volume(newSound,3),
								--[[-----------------------------
									SCA Sound Packs
								-------------------------------]]
								packSelect = Ace3:PackSelect(newSound,4,"SpecificTab",true),
								isSCARandom = Ace3:IsSCARandom(newSound,7,true),
								--[[-----------------------------
									SharedMedia
								-------------------------------]]
								sharedMedia = Ace3:SharedMedia(newSound,4,"SpecificTab",true),
								--[[-----------------------------
									Custom Audio
								-------------------------------]]
								audioSource = Ace3:AudioSource(newSound,4,SOUND_CUSTOM_TYPES,"SpecificTab",true),
								soundEvents = {
									order = 5,
									type = "select",
									name = "Spell Events",
									desc = "Choose whether your custom audio is a file or sound pack",
									disabled = function()
										if (newSound.isSeparate and newSound.validSpell and newSound.numEvents > 1) then
											return false
										else
											return true
										end
									end,
									get = function()
										if (newSound.events.onCast and Ace3:CheckSpell('onCast',newSound.validSpell)) then
											if ((Ace3:GetSelected(newSound) ~= "onCast" and not newSound.events.onSuccess and not newSound.events.onFail and (not newSound.events.onDamage or not Ace3:CheckSpell('onDamage',newSound.validSpell)) and (not newSound.events.onHeal or not Ace3:CheckSpell('onHeal',newSound.validSpell))) or newSound.selected == "empty") then
												newSound.selected = Ace3:SetSelected(newSound,'onCast')
											end
										else
											if (Ace3:GetSelected(newSound) == "onCast") then
												for k,v in pairs(newSound.events) do
													if (v) then
														newSound.selected = Ace3:SetSelected(newSound,k)
														break
													end
												end
												
												if (Ace3:GetSelected(newSound) == "onCast") then
													newSound.selected = "empty"
												end
											end
										end
										
										if (newSound.events.onSuccess and Ace3:CheckSpell('onSuccess',newSound.validSpell)) then
											if ((Ace3:GetSelected(newSound) ~= "onSuccess" and not newSound.events.onCast and not newSound.events.onFail and (not newSound.events.onDamage or not Ace3:CheckSpell('onDamage',newSound.validSpell)) and (not newSound.events.onHeal or not Ace3:CheckSpell('onHeal',newSound.validSpell))) or newSound.selected == "empty") then
												newSound.selected = Ace3:SetSelected(newSound,'onSuccess')
											end
										else
											if (Ace3:GetSelected(newSound) == "onSuccess") then
												for k,v in pairs(newSound.events) do
													if (v) then
														newSound.selected = Ace3:SetSelected(newSound,k)
														break
													end
												end
												
												if (Ace3:GetSelected(newSound) == "onSuccess") then
													newSound.selected = "empty"
												end
											end
										end
										
										if (newSound.events.onFail and Ace3:CheckSpell('onFail',newSound.validSpell)) then
											if ((Ace3:GetSelected(newSound) ~= "onFail" and not newSound.events.onCast and not newSound.events.onSuccess and (not newSound.events.onDamage or not Ace3:CheckSpell('onDamage',newSound.validSpell)) and (not newSound.events.onHeal or not Ace3:CheckSpell('onHeal',newSound.validSpell))) or newSound.selected == "empty") then
												newSound.selected = Ace3:SetSelected(newSound,'onFail')
											end
										else
											if (Ace3:GetSelected(newSound) == "onFail") then
												for k,v in pairs(newSound.events) do
													if (v) then
														newSound.selected = Ace3:SetSelected(newSound,k)
														break
													end
												end
												
												if (Ace3:GetSelected(newSound) == "onFail") then
													newSound.selected = "empty"
												end
											end
										end
										
										if (newSound.events.onDamage and Ace3:CheckSpell('onDamage',newSound.validSpell)) then
											if ((Ace3:GetSelected(newSound) ~= 'onDamage' and not newSound.events.onCast and not newSound.events.onSuccess and not newSound.events.onFail and (not newSound.events.onHeal or not Ace3:CheckSpell('onHeal',newSound.validSpell))) or newSound.selected == "empty") then
												newSound.selected = Ace3:SetSelected(newSound,'onDamage')
											end
										else
											if (Ace3:GetSelected(newSound) == "onDamage") then
												for k,v in pairs(newSound.events) do
													if (v) then
														newSound.selected = Ace3:SetSelected(newSound,k)
														break
													end
												end
												
												if (Ace3:GetSelected(newSound) == "onDamage") then
													newSound.selected = "empty"
												end
											end
										end
										
										if (newSound.events.onHeal and Ace3:CheckSpell('onHeal',newSound.validSpell)) then
											if ((Ace3:GetSelected(newSound) ~= 'onHeal' and not newSound.events.onCast and not newSound.events.onSuccess and not newSound.events.onFail and (not newSound.events.onDamage or not Ace3:CheckSpell('onDamage',newSound.validSpell))) or newSound.selected == "empty") then
												newSound.selected = Ace3:SetSelected(newSound,'onHeal')
											end
										else
											if (Ace3:GetSelected(newSound) == "onHeal") then
												for k,v in pairs(newSound.events) do
													if (v) then
														newSound.selected = Ace3:SetSelected(newSound,k)
														break
													end
												end
												
												if (Ace3:GetSelected(newSound) == "onHeal") then
													newSound.selected = "empty"
												end
											end
										end

										return newSound.selected
									end,
									set = function(self,value)
										newSound.selected = value
										
										newSound.tutorial.flags.spellEvent = true
										
										Ace3:UpdateTutorial(newSound)
									end,
									values = function()
										local args,sortTable = {},{}
										--local numOptions = 0

										--[[for k,v in pairs (newSound.events) do
											if (v) then
												tinsert(sortTable,newSound[k].order,newSound[k].label)
												
											else
												--tinsert(sortTable,db[k].order,"empty")
											end
											numOptions = numOptions + 1
										end]]
										sortTable = Ace3:GetRefreshedTable(newSound,true)

										for i=1,#sortTable do
											if (sortTable[i] ~= "empty") then
												args[i] = sortTable[i]
											end
										end
										
										if (newSound.numEvents == 0) then
											args[0] = "No spell events"
										elseif (Ace3:GetSelected(newSound) == "all") then
											args[0] = "Select a spell event"
										end
										--[[for k,v in pairs(newSound.events) do
											if (v and Ace3:CheckSpell(k,newSound.validSpell)) then
												args[k] = SOUND_EVENTS[k]
											end
										end
										
										if (newSound.numEvents == 0) then
											args["empty"] = "No events"
										end]]
										
										return args
									end
								},
								toggleSeparate = Ace3:ToggleSeparate(newSound,6,'SpecificTab',true),
								audioLocation = Ace3:AudioLocation(newSound,7,true),
								numFiles = Ace3:NumFiles(newSound,8,newSound.errorName,'SpecificTab',true),
								test = Ace3:TestSound(newSound,9,newSound.errorName,'SpecificTab',true),
								isCustomRandom = Ace3:IsCustomRandom(newSound,10),
								reset = Ace3:SoftReset(newSound,11,'SpecificTab',true),
							},
						},
						DamageTriggerHeader = {
							order = 6,
							type = 'group',
							name = 'Damage Triggers',
							guiInline = true,
							hidden = function()
								if (SCA.temp.addSound and newSound.events.onDamage and Ace3:CheckSpell('onDamage',newSound.validSpell)) then
									return false
								else
									return true
								end
							end,
							args = {
								threshold = Ace3:Threshold(newSound,'onDamage','Damage',newSound.errorName,'SpecificTab'),
								critSelect = Ace3:CritSelect(newSound,'onDamage',SOUND_CRIT_TRIGGERS),
							},
						},
						HealingTriggerHeader = {
							order = 7,
							type = 'group',
							name = 'Healing Triggers',
							guiInline = true,
							hidden = function()
								if (SCA.temp.addSound and newSound.events.onHeal and Ace3:CheckSpell('onHeal',newSound.validSpell)) then
									return false
								else
									return true
								end
							end,
							args = {
								threshold = Ace3:Threshold(newSound,'onHeal','Healing',newSound.errorName,'SpecificTab'),
								critSelect = Ace3:CritSelect(newSound,'onHeal',SOUND_CRIT_TRIGGERS),
							},
						},
						submit = {
							order = 8,
							type = "execute",
							name = function()
								if (SCA.temp.editSound) then
									return "Edit Sound"
								else
									return "Add Sound"
								end
							end,
							disabled = function()
								if (newSound.isSeparate) then
									local numValidEvents = 0
									
									for k,v in pairs(newSound.events) do
										if (v and newSound[k].validAudio and Ace3:CheckSpell(k,newSound.validSpell)) then
											numValidEvents = numValidEvents + 1
										end
									end
									
									if (numValidEvents == newSound.numEvents) then
										return false
									else
										return true
									end
								else
									for k,v in pairs(newSound.events) do
										if (v and newSound[k].validAudio and Ace3:CheckSpell(k,newSound.validSpell)) then
											return false
										end
									end
									
									return true
								end
							end,
							hidden = function()
								return not SCA.temp.addSound
							end,
							func = function(self)
								for k,v in pairs(newSound.events) do
									if (v) then
										if ((k ~= "onCast" and k ~= "onSuccess" and k ~= "onFail" and Ace3:CheckSpell(k,newSound.validSpell)) or (k == "onCast" and Ace3:CheckSpell('onCast',newSound.validSpell)) or (k == "onSuccess" and Ace3:CheckSpell('onSuccess',newSound.validSpell)) or (k == "onFail" and Ace3:CheckSpell('onFail',newSound.validSpell))) then
											local _,spellID,spellName = Ace3:CheckSpell('onSuccess',newSound.validSpell)
											local isOverload = 0
											local _,class = UnitClass("player")
											local spec = GetSpecialization()
											
											if (SCA.overloadSpells[class]) then
												for k,v in pairs(SCA.overloadSpells[class][spec]) do
													if (spellName ~= v and strfind(v,spellName)) then
														isOverload = k
														break
													end
												end
											end
											
											if (db.audio.spells.ignored[spellID]) then
												db.audio.spells.ignored[spellID] = nil
											end
											
											if (not specific[spellID]) then
												specific[spellID] = {
													isSeparate = newSound.isSeparate
												}
											end
											
											specific[spellID][k] = {}
											specific[spellID][k].options = {}
											specific[spellID][k].options.source = newSound[k].options.source
											specific[spellID][k].options.channel = newSound[k].options.channel

											if (k == "onDamage") then
												specific[spellID][k].options.trigger = newSound[k].options.trigger
												specific[spellID][k].options.threshold = newSound[k].options.threshold
											end
											
											if (k == "onHeal") then
												specific[spellID][k].options.trigger = newSound[k].options.trigger
												specific[spellID][k].options.threshold = newSound[k].options.threshold
											end
											
											if (newSound[k].options.source == "packs") then
												specific[spellID][k].packs = {}
												specific[spellID][k].packs.value = newSound[k].packs.value
												specific[spellID][k].packs.text = newSound[k].packs.text
												specific[spellID][k].packs.path = newSound[k].packs.path
												specific[spellID][k].packs.max = newSound[k].packs.max
												specific[spellID][k].packs.isRandom = {}
												specific[spellID][k].packs.isRandom.enabled = newSound[k].packs.isRandom.enabled
												specific[spellID][k].packs.isRandom.currentFile = newSound[k].packs.isRandom.currentFile
											elseif (newSound[k].options.source == "shared") then
												specific[spellID][k].shared = {}
												specific[spellID][k].shared.value = newSound[k].shared.value
											elseif (newSound[k].options.source == "custom") then
												specific[spellID][k].custom = {}
												specific[spellID][k].custom.path = newSound[k].custom.path
												specific[spellID][k].custom.max = newSound[k].custom.max
												specific[spellID][k].custom.customType = newSound[k].custom.customType
												specific[spellID][k].custom.isRandom = {}
												specific[spellID][k].custom.isRandom.enabled = newSound[k].custom.isRandom.enabled
												specific[spellID][k].custom.isRandom.currentFile = newSound[k].custom.isRandom.currentFile
											end
											
											if (isOverload > 0) then
												if (not specific[isOverload]) then
													specific[isOverload] = {
														isSeparate = newSound.isSeparate
													}
												end
												
												specific[isOverload][k] = {}
												specific[isOverload][k].options = {}
												specific[isOverload][k].options.source = newSound[k].options.source
												specific[isOverload][k].options.channel = newSound[k].options.channel

												if (k == "onDamage") then
													specific[isOverload][k].options.trigger = newSound[k].options.trigger
													specific[isOverload][k].options.threshold = newSound[k].options.threshold
												end
												
												if (k == "onHeal") then
													specific[isOverload][k].options.trigger = newSound[k].options.trigger
													specific[isOverload][k].options.threshold = newSound[k].options.threshold
												end
												
												if (newSound[k].options.source == "packs") then
													specific[isOverload][k].packs = {}
													specific[isOverload][k].packs.value = newSound[k].packs.value
													specific[isOverload][k].packs.text = newSound[k].packs.text
													specific[isOverload][k].packs.path = newSound[k].packs.path
													specific[isOverload][k].packs.max = newSound[k].packs.max
													specific[isOverload][k].packs.isRandom = {}
													specific[isOverload][k].packs.isRandom.enabled = newSound[k].packs.isRandom.enabled
													specific[isOverload][k].packs.isRandom.currentFile = newSound[k].packs.isRandom.currentFile
												elseif (newSound[k].options.source == "shared") then
													specific[isOverload][k].shared = {}
													specific[isOverload][k].shared.value = newSound[k].shared.value
												elseif (newSound[k].options.source == "custom") then
													specific[isOverload][k].custom = {}
													specific[isOverload][k].custom.path = newSound[k].custom.path
													specific[isOverload][k].custom.max = newSound[k].custom.max
													specific[isOverload][k].custom.customType = newSound[k].custom.customType
													specific[isOverload][k].custom.isRandom = {}
													specific[isOverload][k].custom.isRandom.enabled = newSound[k].custom.isRandom.enabled
													specific[isOverload][k].custom.isRandom.currentFile = newSound[k].custom.isRandom.currentFile
												end
											end
										end
									end
								end
								
								if (SCA.temp.errors.newSound) then
									SCA.temp.errors.newSound = nil
								end
								
								Ace3:ResetDB(newSound,true,true)
								
								self.options.args.SpecificTab.args.soundList.args.messages.name = ' '
								Ace3:ReloadSpellIgnoreList(GetSpellOptions())
								Ace3:ReloadSpellAssignList(GetSpellOptions(),db.audio.spells)
								
								-- The tooltip for this button lingers because the button disappears. This forces to hide it.
								GameTooltip:Hide()
							end,
						},
						spacing = Ace3:Spacer(10,"half"),
						reset = Ace3:HardReset(newSound,"SpecificTab",true),
						assignedSpells = {
							name = "Assigned Spells",
							order = 1,
							desc = "TESTING",
							hidden = function()
								return SCA.temp.addSound
							end,
							type = "group",
							guiInline = true,
							args = {
							},
						},
					},
				},
				ignoreTab = {
					name = IGNORE,
					order = 4,
					type = "group",
					args = {
						input = {
							name = "Spell Input",
							order = 0,
							type = "group",
							desc = "Click on a checkbox to remove a spell from the ignore list.",
							guiInline = true,
							args = {
								spellInfo = {
									type = "input",
									order = 1,				
									name = "Spell ID / Name",
									desc = "Input spell id, or name, to be ignored by the addon.\n\nSpell names and IDs that you input are checked against a pre-built table of spell ID and names. If there is a spell that is missing that you feel should be added, please message me via the addon page.",
									get = function()
											return newIgnore.validSpell
									end,
									set = function(self,value)
										local spellID,name
										local errors = self.options.args.ignoreTab.args.errors.args.messages
										
										Ace3:CheckIgnoreSpell(newIgnore,errors,Ace3:CheckSpell('onSuccess',value))
									end,
								},
								spells = {
									order = 2,
									type = "execute",
									name = "Spells",
									desc = "List of spells, and spell IDs, that pertain to your class and spec.\n\n|cFFFF9B00Orange Spells|r are spells that have sounds assigned to them.\n\nA |cFF00FF00Green Spell|r is the spell that is currently selected.\n\n|cFF999999Grey Spells|r are spells that are already being ignored.",
									func = function()
										local _,class = UnitClass("player")
										local spec = GetSpecialization()
										if (SCA.SpellFrame:IsShown()) then
											SCA.SpellFrame:Hide()
										else
											for k,_ in pairs(SCA.spellNames[class][spec]) do
												if (SCA.SpellFrame[k]) then
													if (ignored[SCA.spellNames[class][spec][k]]) then
														SCA.SpellFrame[k]:Disable()
														SCA.SpellFrame[k]:SetDisabledFontObject(SCA.SpellFrame.dFont)
													elseif (specific[SCA.spellNames[class][spec][k]]) then
														if (not SCA.SpellFrame[k]:IsEnabled()) then
															SCA.SpellFrame[k]:Enable()
														end
														SCA.SpellFrame[k]:SetNormalFontObject(SCA.SpellFrame.cFont)
														SCA.SpellFrame[k]:SetHighlightFontObject(SCA.SpellFrame.CFont)
													elseif (k == SCA.temp.newIgnore.validSpell) then
														if (not SCA.SpellFrame[SCA.temp.newIgnore.validSpell]:IsEnabled()) then
															SCA.SpellFrame[SCA.temp.newIgnore.validSpell]:Enable()
														end
														SCA.SpellFrame[SCA.temp.newIgnore.validSpell]:SetNormalFontObject(SCA.SpellFrame.sFont)
														SCA.SpellFrame[SCA.temp.newIgnore.validSpell]:SetHighlightFontObject(SCA.SpellFrame.sFont)
													else
														if (not SCA.SpellFrame[k]:IsEnabled()) then
															SCA.SpellFrame[k]:Enable()
														end
														SCA.SpellFrame[k]:SetNormalFontObject(SCA.SpellFrame.nFont)
														SCA.SpellFrame[k]:SetHighlightFontObject(SCA.SpellFrame.hFont)
													end
												end
											end
											SCA.SpellFrame:Show()
										end
									end,
									width = "half",
								},
								submit = {
									order = 3,
									type = "execute",
									name = "Ignore",
									disabled = function()
										if (#newIgnore.validSpell > 0) then
											return false
										else
											return true
										end
									end,
									func = function(self)
										local _,spellID = Ace3:CheckSpell('onSuccess',newIgnore.validSpell)

										ignored[spellID] = name
										if (specific[spellID]) then
											specific[spellID] = nil
											Ace3:ReloadSpellAssignList(GetSpellOptions(),db.audio.spells)
										end
										
										self.options.args.ignoreTab.args.errors.args.messages.name = " "
										newIgnore.isValidSpell = false
										newIgnore.validSpell = ''
										ignored[spellID] = true

										if (SCA.temp.errors.ignored) then
											SCA.temp.errors.ignored = nil
										end
								
										Ace3:ReloadSpellIgnoreList(GetSpellOptions())
									end,
								},
							},
						},
						errors = Ace3:Errors(newIgnore.errorName,4),
						ignoreList = {
							name = "Ignored Spells",
							order = 5,
							type = "group",
							dialogInline = true,
							args = {
							},
						},
					},
				},
			},
		}
	end
	
	return spell_options
end

local defense_options

local function GetDefenseOptions()
	SCA.temp.selectedMenuItem = 3
	
	if (SCA.SpellFrame and SCA.SpellFrame:IsShown()) then
		SCA.SpellFrame:Hide()
	end
	
	if (not defense_options) then
		local db = Ace3.db.char
		local defense = db.audio.defense
		
		defense_options = {
			type = "group",
			childGroups = "tab",
			order = 1,
			name = "",
			desc = '',
			args = {
				disabledError = Ace3:DisabledError(defense,0),
				options = {
					name = "Toggle Options",
					order = 1,
					type = "group",
					guiInline = true,
					args = Ace3:BuildToggleCheckboxes(defense,nil,true),
				},
				soundList = Ace3:SoundList(defense,2),
				errors = Ace3:Errors(defense.errorName,3),
				SoundConfig = {
					name = "Audio Config",
					order = 4,
					type = "group",
					guiInline = true,
					args = {
						source = Ace3:Source(defense,1,defense.errorName,SOUND_SOURCE_OPTIONS,nil,true),
						channel = Ace3:Channel(defense,2,SOUND_CHANNEL_OPTIONS),
						volume = Ace3:Volume(defense,3),
						--[[-----------------------------
							SCA Sound Packs
						-------------------------------]]
						packSelect = Ace3:PackSelect(defense,4),
						isSCARandom = Ace3:IsSCARandom(defense,7),
						--[[-----------------------------
							SharedMedia
						-------------------------------]]
						sharedMedia = Ace3:SharedMedia(defense,4),
						--[[-----------------------------
							Custom Audio
						-------------------------------]]
						audioSource = Ace3:AudioSource(defense,4,SOUND_CUSTOM_TYPES,nil,nil,true),
						events = {
							order = 5,
							type = "select",
							name = "Defense Event",
							desc = "Choose which event you want to edit",
							disabled = function()
								if (defense.isSeparate) then
									return false
								else
									return true
								end
							end,
							get = function()
								return defense.selected
							end,
							set = function(self,value)
								defense.selected = value
							end,
							values = function()
								local args,sortTable = {},{}

								sortTable = Ace3:GetRefreshedTable(defense)
								--[[for k,_ in pairs (defense.events) do
									tinsert(sortTable,defense[k].order,defense[k].label)
								end]]
								
								for i=1,#sortTable do
									if (sortTable[i] ~= "empty") then
										args[i] = sortTable[i]
									end
								end
								
								if (defense.numEvents == 0) then
									args[0] = "No defense events"
								elseif (Ace3:GetSelected(defense) == "all") then
									args[0] = 'Select a defense event'
								end
									
								wipe(sortTable)

								return args
							end,
						},
						toggleSeparate = Ace3:ToggleSeparate(defense,6,nil,nil,true,true),
						audioLocation = Ace3:AudioLocation(defense,7,nil,true),
						numFiles = Ace3:NumFiles(defense,8,defense.errorName,nil,nil,true),
						test = Ace3:TestSound(defense,9,defense.errorName,nil,nil,true),
						isCustomRandom = Ace3:IsCustomRandom(defense,10),
						--spacing1 = Ace3:Spacer(6,"half"),
						
					},
				},	
			},
		}
		
	end
	
	return defense_options
end

local combat_options

local function GetCombatOptions()
	SCA.temp.selectedMenuItem = 4
	
	if (SCA.SpellFrame and SCA.SpellFrame:IsShown()) then
		SCA.SpellFrame:Hide()
	end
	
	if (not combat_options) then
		local db = Ace3.db.char
		local combat = db.audio.combat
		
		combat_options = {
			type = "group",
			childGroups = "tab",
			order = 1,
			name = "",
			desc = '',
			args = Ace3:CombatTabBuilder(),
		}
		
	end
	
	return combat_options
end

local loot_options

local function GetLootOptions()
	SCA.temp.selectedMenuItem = 5
	if (not loot_options) then
		--local db = Ace3.db.char
		local db = Ace3.db.char
		
		local damage = db.audio.spells.damage
		local heals = db.audio.spells.heals
		local specific = db.audio.spells.specific
		local ignored = db.audio.spells.ignored
		
		local newSound = SCA.temp.newSound
		local newIgnore = SCA.temp.newIgnore
		
		loot_options = {
			type = "group",
			childGroups = "tab",
			order = 1,
			name = "",
			desc = '',
			args = Ace3:LootTabBuilder(),
		}
	end
	
	return loot_options
end

local chat_options

local function GetChatOptions()
	SCA.temp.selectedMenuItem = 6
	
	if (SCA.SpellFrame and SCA.SpellFrame:IsShown()) then
		SCA.SpellFrame:Hide()
	end
	
	if (not chat_options) then
		chat_options = {
			type = "group",
			childGroups = "tab",
			order = 1,
			name = "",
			desc = '',
			args = Ace3:ChatTabBuilder(),
		}
	end
	
	return chat_options
end

local move_options

local function GetMoveOptions()
	SCA.temp.selectedMenuItem = 7
	
	if (SCA.SpellFrame and SCA.SpellFrame:IsShown()) then
		SCA.SpellFrame:Hide()
	end
	
	if (not move_options) then
		move_options = {
			type = "group",
			childGroups = "tab",
			order = 1,
			name = "",
			desc = '',
			args = Ace3:MoveTabBuilder(),
		}
	end
	
	return move_options
end

function Ace3:SetupOptions()
	local ACFG = LibStub("AceConfig-3.0")
	ACFG:RegisterOptionsTable("CustomAudio Quick Toggle", GetToggleOptions)
	ACFG:RegisterOptionsTable("CustomAudio Spell Sounds", GetSpellOptions)
	ACFG:RegisterOptionsTable("CustomAudio Defense Sounds", GetDefenseOptions)
	ACFG:RegisterOptionsTable("CustomAudio Combat Sounds", GetCombatOptions)
	ACFG:RegisterOptionsTable("CustomAudio Loot Sounds", GetLootOptions)
	ACFG:RegisterOptionsTable("CustomAudio Chat Sounds", GetChatOptions)
	ACFG:RegisterOptionsTable("CustomAudio Movement Sounds", GetMoveOptions)

	local ACD = LibStub("AceConfigDialog-3.0")
	ACD:AddToBlizOptions("CustomAudio Quick Toggle", "Quick Toggle", "CustomAudio")
	ACD:AddToBlizOptions("CustomAudio Spell Sounds", "Spell Sounds", "CustomAudio")
	ACD:AddToBlizOptions("CustomAudio Defense Sounds", "Defense Sounds", "CustomAudio")
	ACD:AddToBlizOptions("CustomAudio Combat Sounds", "Combat Sounds", "CustomAudio")
	ACD:AddToBlizOptions("CustomAudio Loot Sounds", "Loot Sounds", "CustomAudio")
	ACD:AddToBlizOptions("CustomAudio Chat Sounds", "Chat Sounds", "CustomAudio")
	ACD:AddToBlizOptions("CustomAudio Movement Sounds", "Movement Sounds", "CustomAudio")
end


InterfaceOptionsFrame:HookScript("OnUpdate",function(self)
	-- The purpose here is to refresh the contents of the addon's config panel; simulating a click to the left navigation items will do this.
	-- This will only be done if the SHIFT key is down (making sure to check if it's down only once, via the "isShiftDown" variable).
	-- We also want to make sure the keyboard doesn't have focus, as SHIFT is commonly pressed while chatting.
	if (IsShiftKeyDown() and not SCA.temp.isShiftDown and not GetCurrentKeyBoardFocus()) then
		Ace3:NavigateInterfaceOptions(SCA.temp.selectedMenuItem,true)
		SCA.temp.isShiftDown = true
	elseif (not IsShiftKeyDown() and SCA.temp.isShiftDown) then
		Ace3:NavigateInterfaceOptions(SCA.temp.selectedMenuItem,true)
		SCA.temp.isShiftDown = false
	end
end)

InterfaceOptionsFrame:HookScript("OnShow",function(self)
	Ace3:UpdateInterfaceSettings()
end)

InterfaceOptionsFrame:HookScript("OnHide",function(self)
	if (SCA.SpellFrame:IsShown()) then
		SCA.SpellFrame:Hide()
	end
end)